@@ -19,7 +19,8 @@
 // CXWindowsClientTaskBarReceiver
 //
 
-CXWindowsClientTaskBarReceiver::CXWindowsClientTaskBarReceiver()
+CXWindowsClientTaskBarReceiver::CXWindowsClientTaskBarReceiver(
+				const CBufferedLogOutputter*)
 {
 	// add ourself to the task bar
 	ARCH->addReceiver(this);
@@ -53,9 +54,3 @@ CXWindowsClientTaskBarReceiver::getIcon() const
 {
 	return NULL;
 }
-
-void
-CXWindowsClientTaskBarReceiver::onStatusChanged()
-{
-	// do nothing
-}
