@@ -157,6 +157,9 @@ CMSWindowsServerTaskBarReceiver::runMenu(int x, int y)
 	SetForegroundWindow(m_window);
 	HMENU menu = GetSubMenu(m_menu, 0);
 	SetMenuDefaultItem(menu, IDC_TASKBAR_STATUS, FALSE);
+	HMENU logLevelMenu = GetSubMenu(menu, 3);
+	CheckMenuRadioItem(logLevelMenu, 0, 6,
+							CLOG->getFilter() - CLog::kERROR, MF_BYPOSITION);
 	int n = TrackPopupMenu(menu,
 							TPM_NONOTIFY |
 							TPM_RETURNCMD |
@@ -175,6 +178,10 @@ CMSWindowsServerTaskBarReceiver::runMenu(int x, int y)
 		copyLog();
 		break;
 
+	case IDC_TASKBAR_SHOW_LOG:
+		ARCH->showConsole(true);
+		break;
+
 	case IDC_RELOAD_CONFIG:
 		EVENTQUEUE->addEvent(CEvent(getReloadConfigEvent(),
 							IEventQueue::getSystemTarget()));
@@ -185,6 +192,34 @@ CMSWindowsServerTaskBarReceiver::runMenu(int x, int y)
 							IEventQueue::getSystemTarget()));
 		break;
 
+	case IDC_TASKBAR_LOG_LEVEL_ERROR:
+		CLOG->setFilter(CLog::kERROR);
+		break;
+
+	case IDC_TASKBAR_LOG_LEVEL_WARNING:
+		CLOG->setFilter(CLog::kWARNING);
+		break;
+
+	case IDC_TASKBAR_LOG_LEVEL_NOTE:
+		CLOG->setFilter(CLog::kNOTE);
+		break;
+
+	case IDC_TASKBAR_LOG_LEVEL_INFO:
+		CLOG->setFilter(CLog::kINFO);
+		break;
+
+	case IDC_TASKBAR_LOG_LEVEL_DEBUG:
+		CLOG->setFilter(CLog::kDEBUG);
+		break;
+
+	case IDC_TASKBAR_LOG_LEVEL_DEBUG1:
+		CLOG->setFilter(CLog::kDEBUG1);
+		break;
+
+	case IDC_TASKBAR_LOG_LEVEL_DEBUG2:
+		CLOG->setFilter(CLog::kDEBUG2);
+		break;
+
 	case IDC_TASKBAR_QUIT:
 		quit();
 		break;
