@@ -19,13 +19,21 @@
 #define IDC_TASKBAR_LOG                 40005
 #define IDC_RELOAD_CONFIG               40006
 #define IDC_FORCE_RECONNECT             40007
+#define IDC_TASKBAR_SHOW_LOG            40008
+#define IDC_TASKBAR_LOG_LEVEL_ERROR     40009
+#define IDC_TASKBAR_LOG_LEVEL_WARNING   40010
+#define IDC_TASKBAR_LOG_LEVEL_NOTE      40011
+#define IDC_TASKBAR_LOG_LEVEL_INFO      40012
+#define IDC_TASKBAR_LOG_LEVEL_DEBUG     40013
+#define IDC_TASKBAR_LOG_LEVEL_DEBUG1    40014
+#define IDC_TASKBAR_LOG_LEVEL_DEBUG2    40015
 
 // Next default values for new objects
 // 
 #ifdef APSTUDIO_INVOKED
 #ifndef APSTUDIO_READONLY_SYMBOLS
 #define _APS_NEXT_RESOURCE_VALUE        109
-#define _APS_NEXT_COMMAND_VALUE         40008
+#define _APS_NEXT_COMMAND_VALUE         40016
 #define _APS_NEXT_CONTROL_VALUE         1003
 #define _APS_NEXT_SYMED_VALUE           101
 #endif
