@@ -19,7 +19,7 @@
 // CArchConsoleUnix
 //
 
-CArchConsoleUnix::CArchConsoleUnix()
+CArchConsoleUnix::CArchConsoleUnix(void*)
 {
 	// do nothing
 }
@@ -41,6 +41,12 @@ CArchConsoleUnix::closeConsole()
 	// do nothing
 }
 
+void
+CArchConsoleUnix::showConsole(bool)
+{
+	// do nothing
+}
+
 void
 CArchConsoleUnix::writeConsole(const char* str)
 {
