@@ -19,6 +19,7 @@
 
 #include "IArchConsole.h"
 #include "IArchMultithread.h"
+#include "stddeque.h"
 #include <windows.h>
 
 #define ARCH_CONSOLE CArchConsoleWindows
@@ -26,23 +27,51 @@
 //! Win32 implementation of IArchConsole
 class CArchConsoleWindows : public IArchConsole {
 public:
-	CArchConsoleWindows();
+	CArchConsoleWindows(void*);
 	virtual ~CArchConsoleWindows();
 
 	// IArchConsole overrides
 	virtual void		openConsole(const char* title);
 	virtual void		closeConsole();
+	virtual void		showConsole(bool showIfEmpty);
 	virtual void		writeConsole(const char*);
 	virtual const char*	getNewlineForConsole();
 
 private:
-	static BOOL WINAPI	signalHandler(DWORD);
+	void				clearBuffer();
+	void				appendBuffer(const char*);
+	void				setSize(int width, int height);
+
+	LRESULT				wndProc(HWND, UINT, WPARAM, LPARAM);
+	static LRESULT CALLBACK
+						staticWndProc(HWND, UINT, WPARAM, LPARAM);
+	void				threadMainLoop();
+	static void*		threadEntry(void*);
 
 private:
-	static CArchThread	s_thread;
+	typedef std::deque<std::string> MessageBuffer;
 
+	static CArchConsoleWindows*	s_instance;
+	static HINSTANCE	s_appInstance;
+
+	// multithread data
 	CArchMutex			m_mutex;
-	HANDLE				m_output;
+	CArchCond			m_condVar;
+	bool				m_ready;
+	CArchThread			m_thread;
+
+	// child thread data
+	HWND				m_frame;
+	HWND				m_hwnd;
+	LONG				m_wChar;
+	LONG				m_hChar;
+	bool				m_show;
+
+	// messages
+	size_t				m_maxLines;
+	size_t				m_maxCharacters;
+	size_t				m_numCharacters;
+	MessageBuffer		m_buffer;
 };
 
 #endif
