@@ -56,6 +56,18 @@ public:
 	*/
 	static bool			isWindowsModern();
 
+	//! Set the application icons
+	/*!
+	Set the application icons.
+	*/
+	static void			setIcons(HICON largeIcon, HICON smallIcon);
+
+	//! Get the application icons
+	/*!
+	Get the application icons.
+	*/
+	static void			getIcons(HICON& largeIcon, HICON& smallIcon);
+
 	//! Run the daemon
 	/*!
 	Delegates to CArchDaemonWindows.
@@ -86,6 +98,12 @@ public:
 	//! Open and return a registry key, closing the parent key
 	static HKEY			openKey(HKEY parent, const TCHAR* const* keyPath);
 
+	//! Open/create and return a registry key, closing the parent key
+	static HKEY			addKey(HKEY parent, const TCHAR* child);
+
+	//! Open/create and return a registry key, closing the parent key
+	static HKEY			addKey(HKEY parent, const TCHAR* const* keyPath);
+
 	//! Close a key
 	static void			closeKey(HKEY);
 
@@ -144,6 +162,13 @@ public:
 	static void			removeBusyState(DWORD busyModes);
 
 private:
+	//! Open and return a registry key, closing the parent key
+	static HKEY			openKey(HKEY parent, const TCHAR* child, bool create);
+
+	//! Open and return a registry key, closing the parent key
+	static HKEY			openKey(HKEY parent, const TCHAR* const* keyPath,
+							bool create);
+
 	//! Read a string value from the registry
 	static std::string	readBinaryOrString(HKEY, const TCHAR* name, DWORD type);
 
@@ -159,6 +184,8 @@ private:
 	static CDialogs*	s_dialogs;
 	static DWORD		s_busyState;
 	static STES_t		s_stes;
+	static HICON		s_largeIcon;
+	static HICON		s_smallIcon;
 };
 
 #endif
