@@ -115,55 +115,38 @@ CStringUtil::vformat(const char* fmt, va_list args)
 
 CString
 CStringUtil::print(const char* fmt, ...)
-{
-	va_list args;
-	va_start(args, fmt);
-	CString result = vprint(fmt, args);
-	va_end(args);
-	return result;
-}
-
-CString
-CStringUtil::vprint(const char* fmt, va_list args)
 {
 	char tmp[1024];
-	char* buffer = vsprint(tmp, sizeof(tmp) / sizeof(tmp[0]), 0, 0, fmt, args);
-	if (buffer == tmp) {
-		return buffer;
-	}
-	else {
-		CString result(buffer);
-		delete[] buffer;
-		return result;
-	}
-}
+	char* buffer = tmp;
+	int len      = (int)(sizeof(tmp) / sizeof(tmp[0]));
+	CString result;
+	while (buffer != NULL) {
+		// try printing into the buffer
+		va_list args;
+		va_start(args, fmt);
+		int n = ARCH->vsnprintf(buffer, len, fmt, args);
+		va_end(args);
 
-char*
-CStringUtil::vsprint(char* buffer, int len,
-				int prefix, int suffix, const char* fmt, va_list args)
-{
-	assert(len > 0);
+		// if the buffer wasn't big enough then make it bigger and try again
+		if (n < 0 || n > len) {
+			if (buffer != tmp) {
+				delete[] buffer;
+			}
+			len   *= 2;
+			buffer = new char[len];
+		}
 
-	// try writing to input buffer
-	int n;
-	if (buffer != NULL && len >= prefix + suffix) {
-		n = ARCH->vsnprintf(buffer + prefix,
-							len - (prefix + suffix), fmt, args);
-		if (n >= 0 && n <= len - (prefix + suffix))
-			return buffer;
+		// if it was big enough then save the string and don't try again
+		else {
+			result = buffer;
+			if (buffer != tmp) {
+				delete[] buffer;
+			}
+			buffer = NULL;
+		}
 	}
 
-	// start allocating buffers until we write the whole string
-	buffer = NULL;
-	do {
-		delete[] buffer;
-		len *= 2;
-		buffer = new char[len + (prefix + suffix)];
-		n = ARCH->vsnprintf(buffer + prefix,
-							len - (prefix + suffix), fmt, args);
-	} while (n < 0 || n > len - (prefix + suffix));
-
-	return buffer;
+	return result;
 }
 
 
