@@ -18,6 +18,9 @@
 #include "IListenSocket.h"
 #include "IArchNetwork.h"
 
+class CMutex;
+class ISocketMultiplexerJob;
+
 //! TCP listen socket
 /*!
 A listen socket using TCP.
@@ -30,12 +33,19 @@ public:
 	// ISocket overrides
 	virtual void		bind(const CNetworkAddress&);
 	virtual void		close();
+	virtual void*		getEventTarget() const;
 
 	// IListenSocket overrides
 	virtual IDataSocket*	accept();
 
+private:
+	ISocketMultiplexerJob*
+						serviceListening(ISocketMultiplexerJob*,
+							bool, bool, bool);
+
 private:
 	CArchSocket			m_socket;
+	CMutex*				m_mutex;
 };
 
 #endif
