@@ -38,6 +38,9 @@
 #define SYNERGY_MSG_INPUT_LAST		SYNERGY_MSG_PRE_WARP
 #define SYNERGY_HOOK_LAST_MSG		SYNERGY_MSG_DEBUG
 
+#define SYNERGY_HOOK_FAKE_INPUT_VIRTUAL_KEY	VK_CANCEL
+#define SYNERGY_HOOK_FAKE_INPUT_SCANCODE	0
+
 extern "C" {
 
 enum EHookResult {
