@@ -49,6 +49,10 @@ public:
 	// IPrimaryScreen overrides
 	virtual void		reconfigure(UInt32 activeSides);
 	virtual void		warpCursor(SInt32 x, SInt32 y);
+	virtual UInt32		registerHotKey(KeyID key, KeyModifierMask mask);
+	virtual void		unregisterHotKey(UInt32 id);
+	virtual void		fakeInputBegin();
+	virtual void		fakeInputEnd();
 	virtual SInt32		getJumpZoneSize() const;
 	virtual bool		isAnyMouseButtonDown() const;
 	virtual void		getCursorCenter(SInt32& x, SInt32& y) const;
@@ -57,7 +61,7 @@ public:
 	virtual void		fakeMouseButton(ButtonID id, bool press) const;
 	virtual void		fakeMouseMove(SInt32 x, SInt32 y) const;
 	virtual void		fakeMouseRelativeMove(SInt32 dx, SInt32 dy) const;
-	virtual void		fakeMouseWheel(SInt32 delta) const;
+	virtual void		fakeMouseWheel(SInt32 xDelta, SInt32 yDelta) const;
 
 	// IPlatformScreen overrides
 	virtual void		enable();
@@ -112,7 +116,7 @@ private:
 		KeyCode			m_keycode;
 	};
 
-	Display*			openDisplay(const char* displayName) const;
+	Display*			openDisplay(const char* displayName);
 	void				saveShape();
 	Window				openWindow() const;
 	void				openIM();
@@ -120,6 +124,7 @@ private:
 	bool				grabMouseAndKeyboard();
 	void				onKeyPress(XKeyEvent&);
 	void				onKeyRelease(XKeyEvent&, bool isRepeat);
+	bool				onHotKey(XKeyEvent&, bool isRepeat);
 	void				onMousePress(const XButtonEvent&);
 	void				onMouseRelease(const XButtonEvent&);
 	void				onMouseMove(const XMotionEvent&);
@@ -133,10 +138,27 @@ private:
 
 	void				warpCursorNoFlush(SInt32 x, SInt32 y);
 
+	void				refreshKeyboard(XEvent*);
+
 	static Bool			findKeyEvent(Display*, XEvent* xevent, XPointer arg);
 
 private:
+	struct CHotKeyItem {
+	public:
+		CHotKeyItem(int, unsigned int);
+
+		bool			operator<(const CHotKeyItem&) const;
+
+	private:
+		int				m_keycode;
+		unsigned int	m_mask;
+	};
 	typedef std::set<bool> CFilteredKeycodes;
+	typedef std::vector<std::pair<int, unsigned int> > HotKeyList;
+	typedef std::map<UInt32, HotKeyList> HotKeyMap;
+	typedef std::vector<UInt32> HotKeyIDList;
+	typedef std::map<CHotKeyItem, UInt32> HotKeyToIDMap;
+
 	// true if screen is being used as a primary screen, false otherwise
 	bool				m_isPrimary;
 
@@ -158,6 +180,11 @@ private:
 	// keyboard stuff
 	CXWindowsKeyState*	m_keyState;
 
+	// hot key stuff
+	HotKeyMap			m_hotKeys;
+	HotKeyIDList		m_oldHotKeyIDs;
+	HotKeyToIDMap		m_hotKeyToIDMap;
+
 	// input focus stuff
 	Window				m_lastFocus;
 	int					m_lastFocusRevert;
@@ -190,6 +217,10 @@ private:
 	bool				m_xtestIsXineramaUnaware;
 	bool				m_xinerama;
 
+	// XKB extension stuff
+	bool				m_xkb;
+	int					m_xkbEventBase;
+
 	// pointer to (singleton) screen.  this is only needed by
 	// ioErrorHandler().
 	static CXWindowsScreen*	s_screen;
